@@ -1,7 +1,7 @@
 /*
  * dfu-util
  *
- * (C) 2007 by OpenMoko, Inc.
+ * (C) 2007-2008 by OpenMoko, Inc.
  * Written by Harald Welte <laforge@openmoko.org>
  *
  * Based on existing code of dfu-programmer-0.4
@@ -21,13 +21,13 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include <stdlib.h>
 #include <stdio.h>
+#include <unistd.h>
 #include <string.h>
 #include <getopt.h>
 #include <usb.h>
 #include <errno.h>
-#include <byteswap.h>
-#include <endian.h>
 
 #include "dfu.h"
 #include "usb_dfu.h"
@@ -41,17 +41,13 @@
 #include <usbpath.h>
 #endif
 
-/* define a portable macro for swapping a 16bit word */
-#if defined(WORDS_BIGENDIAN)
-# if defined(__APPLE__) && defined (OSX)
-#  include <libkern/OSByteOrder.h>
-#  define LE2CPU16(x)	OSSwapInt16(x)
-# else
-#  define LE2CPU16(x)	bswap_16(x)
-# endif
-#else
-# define LE2CPU16(x)	(x)
-#endif
+/* define a portable function for reading a 16bit little-endian word */
+unsigned short get_int16_le(const void *p)
+{
+    const unsigned char *cp = p;
+
+    return ( cp[0] ) | ( ((unsigned short)cp[1]) << 8 );
+}
 
 int debug;
 static int verbose = 0;
@@ -168,15 +164,20 @@ static int print_dfu_if(struct dfu_if *dfu_if, void *v)
 	int if_name_str_idx;
 	char name[MAX_STR_LEN+1] = "UNDEFINED";
 
-	if_name_str_idx = dev->config[dfu_if->configuration].interface[dfu_if->interface].altsetting[dfu_if->altsetting].iInterface;
+	if_name_str_idx = dev->config[dfu_if->configuration]
+				.interface[dfu_if->interface]
+				.altsetting[dfu_if->altsetting].iInterface;
 	if (if_name_str_idx) {
 		if (!dfu_if->dev_handle)
 			dfu_if->dev_handle = usb_open(dfu_if->dev);
 		if (dfu_if->dev_handle)
-			usb_get_string_simple(dfu_if->dev_handle, if_name_str_idx, name, MAX_STR_LEN);
+			usb_get_string_simple(dfu_if->dev_handle,
+					      if_name_str_idx, name,
+					      MAX_STR_LEN);
 	}
 
-	printf("Found %s: [0x%04x:0x%04x] devnum=%u, cfg=%u, intf=%u, alt=%u, name=\"%s\"\n", 
+	printf("Found %s: [0x%04x:0x%04x] devnum=%u, cfg=%u, intf=%u, "
+	       "alt=%u, name=\"%s\"\n", 
 	       dfu_if->flags & DFU_IFF_DFU ? "DFU" : "Runtime",
 	       dev->descriptor.idVendor, dev->descriptor.idProduct,
 	       dev->devnum, dfu_if->configuration, dfu_if->interface,
@@ -434,9 +435,11 @@ int main(int argc, char **argv)
 	int page_size = getpagesize();
 	int ret;
 	
-	printf("dfu-util - (C) 2007 by OpenMoko Inc.\n"
+	printf("dfu-util - (C) 2007-2008 by OpenMoko Inc.\n"
 	       "This program is Free Software and has ABSOLUTELY NO WARRANTY\n\n");
 
+	printf("dfu-util does currently only support DFU version 1.0\n\n");
+
 	memset(dif, 0, sizeof(*dif));
 
 	usb_init();
@@ -446,7 +449,8 @@ int main(int argc, char **argv)
 
 	while (1) {
 		int c, option_index = 0;
-		c = getopt_long(argc, argv, "hVvld:p:c:i:a:t:U:D:R", opts, &option_index);
+		c = getopt_long(argc, argv, "hVvld:p:c:i:a:t:U:D:R", opts,
+				&option_index);
 		if (c == -1)
 			break;
 
@@ -460,6 +464,9 @@ int main(int argc, char **argv)
 			exit(0);
 			break;
 		case 'v':
+			if (verbose) {
+				usb_set_debug(255);
+			}
 			verbose = 1;
 			break;
 		case 'l':
@@ -469,7 +476,8 @@ int main(int argc, char **argv)
 		case 'd':
 			/* Parse device */
 			if (parse_vendprod(&vendprod, optarg) < 0) {
-				fprintf(stderr, "unable to parse `%s'\n", optarg);
+				fprintf(stderr, "unable to parse `%s' as a vendor:product\n", optarg);
+
 				exit(2);
 			}
 			dif->vendor = vendprod.vendor;
@@ -535,7 +543,7 @@ int main(int argc, char **argv)
 	}
 
 	if (!filename) {
-		fprintf(stderr, "You need to specify a filename to -D -r -U\n");
+		fprintf(stderr, "You need to specify a filename to -D or -U\n");
 		help();
 		exit(2);
 	}
@@ -559,7 +567,7 @@ int main(int argc, char **argv)
 	if (!get_first_dfu_device(dif))
 		exit(3);
 
-	/* We have exactly one device. It's usb_device is now in dif->dev */
+	/* We have exactly one device. Its usb_device is now in dif->dev */
 
 	printf("Opening USB Device 0x%04x:0x%04x...\n", dif->vendor, dif->product);
 	dif->dev_handle = usb_open(dif->dev);
@@ -581,43 +589,60 @@ int main(int argc, char **argv)
 
 		printf("Claiming USB DFU Runtime Interface...\n");
 		if (usb_claim_interface(_rt_dif.dev_handle, _rt_dif.interface) < 0) {
-			fprintf(stderr, "Cannot claim interface: %s\n", usb_strerror());
+			fprintf(stderr, "Cannot claim interface %d: %s\n",
+				_rt_dif.interface,
+				usb_strerror());
+			exit(1);
+		}
+
+		if (usb_set_altinterface(_rt_dif.dev_handle, 0) < 0) {
+			fprintf(stderr, "Cannot set alt interface: %s\n",
+				usb_strerror());
 			exit(1);
 		}
 
 		printf("Determining device status: ");
 		if (dfu_get_status(_rt_dif.dev_handle, _rt_dif.interface, &status ) < 0) {
-			fprintf(stderr, "error get_status: %s\n", usb_strerror());
+			fprintf(stderr, "error get_status: %s\n",
+				usb_strerror());
 			exit(1);
 		}
-		printf("state = %s, status = %d\n", dfu_state_to_string(status.bState), status.bStatus);
+		printf("state = %s, status = %d\n", 
+		       dfu_state_to_string(status.bState), status.bStatus);
 
 		switch (status.bState) {
 		case DFU_STATE_appIDLE:
 		case DFU_STATE_appDETACH:
-			printf("Device really in Runtime Mode, send DFU detach request...\n");
-			if (dfu_detach(_rt_dif.dev_handle, _rt_dif.interface, 1000) < 0) {
-				fprintf(stderr, "error detaching: %s\n", usb_strerror());
+			printf("Device really in Runtime Mode, send DFU "
+			       "detach request...\n");
+			if (dfu_detach(_rt_dif.dev_handle, 
+				       _rt_dif.interface, 1000) < 0) {
+				fprintf(stderr, "error detaching: %s\n",
+					usb_strerror());
 				exit(1);
 				break;
 			}
 			printf("Resetting USB...\n");
 			ret = usb_reset(_rt_dif.dev_handle);
 			if (ret < 0 && ret != -ENODEV)
-				fprintf(stderr, "error resetting after detach: %s\n", 
+				fprintf(stderr,
+					"error resetting after detach: %s\n", 
 					usb_strerror());
 			sleep(2);
 			break;
 		case DFU_STATE_dfuERROR:
 			printf("dfuERROR, clearing status\n");
-			if (dfu_clear_status(_rt_dif.dev_handle, _rt_dif.interface) < 0) {
-				fprintf(stderr, "error clear_status: %s\n", usb_strerror());
+			if (dfu_clear_status(_rt_dif.dev_handle,
+					     _rt_dif.interface) < 0) {
+				fprintf(stderr, "error clear_status: %s\n",
+					usb_strerror());
 				exit(1);
 				break;
 			}
 			break;
 		default:
-			fprintf(stderr, "WARNING: Runtime device already in DFU state ?!?\n");
+			fprintf(stderr, "WARNING: Runtime device already "
+				"in DFU state ?!?\n");
 			goto dfustate;
 			break;
 		}
@@ -646,9 +671,9 @@ int main(int argc, char **argv)
 			fprintf(stderr, "Lost device after RESET?\n");
 			exit(1);
 		} else if (num_devs > 1) {
-			fprintf(stderr, "More than one DFU capable USB device found, "
-			       "you might try `--list' and then disconnect all but one "
-			       "device\n");
+			fprintf(stderr, "More than one DFU capable USB "
+				"device found, you might try `--list' and "
+				"then disconnect all but one device\n");
 			exit(1);
 		}
 		if (!get_first_dfu_device(dif))
@@ -657,7 +682,8 @@ int main(int argc, char **argv)
 		printf("Opening USB Device...\n");
 		dif->dev_handle = usb_open(dif->dev);
 		if (!dif->dev_handle) {
-			fprintf(stderr, "Cannot open device: %s\n", usb_strerror());
+			fprintf(stderr, "Cannot open device: %s\n",
+				usb_strerror());
 			exit(1);
 		}
 	} else {
@@ -690,10 +716,11 @@ dfustate:
 		exit(1);
 	} else if (num_ifs == 1) {
 		if (!get_first_dfu_if(dif)) {
-			fprintf(stderr, "Can't find the single available DFU IF\n");
+			fprintf(stderr, "Can't find the single available "
+				"DFU IF\n");
 			exit(1);
 		}
-	} else if (num_ifs > 1 && !dif->flags & (DFU_IFF_IFACE|DFU_IFF_ALT)) {
+	} else if (num_ifs > 1 && !(dif->flags & (DFU_IFF_IFACE|DFU_IFF_ALT))) {
 		fprintf(stderr, "We have %u DFU Interfaces/Altsettings, "
 			"you have to specify one via --intf / --alt options\n",
 			num_ifs);
@@ -703,17 +730,19 @@ dfustate:
 #if 0
 	printf("Setting Configuration %u...\n", dif->configuration);
 	if (usb_set_configuration(dif->dev_handle, dif->configuration) < 0) {
-		fprintf(stderr, "Cannot set configuration: %s\n", usb_strerror());
+		fprintf(stderr, "Cannot set configuration: %s\n",
+			usb_strerror());
 		exit(1);
 	}
 #endif
 	printf("Claiming USB DFU Interface...\n");
 	if (usb_claim_interface(dif->dev_handle, dif->interface) < 0) {
-		fprintf(stderr, "Cannot claim interface: %s\n", usb_strerror());
+		fprintf(stderr, "Cannot claim interface: %s\n",
+			usb_strerror());
 		exit(1);
 	}
 
-	printf("Setting Alternate Setting ...\n");
+	printf("Setting Alternate Setting #%d ...\n", dif->altsetting);
 	if (usb_set_altinterface(dif->dev_handle, dif->altsetting) < 0) {
 		fprintf(stderr, "Cannot set alternate interface: %s\n",
 			usb_strerror());
@@ -726,7 +755,8 @@ status_again:
 		fprintf(stderr, "error get_status: %s\n", usb_strerror());
 		exit(1);
 	}
-	printf("state = %s, status = %d\n", dfu_state_to_string(status.bState), status.bStatus);
+	printf("state = %s, status = %d\n",
+	       dfu_state_to_string(status.bState), status.bStatus);
 
 	switch (status.bState) {
 	case DFU_STATE_appIDLE:
@@ -737,7 +767,8 @@ status_again:
 	case DFU_STATE_dfuERROR:
 		printf("dfuERROR, clearing status\n");
 		if (dfu_clear_status(dif->dev_handle, dif->interface) < 0) {
-			fprintf(stderr, "error clear_status: %s\n", usb_strerror());
+			fprintf(stderr, "error clear_status: %s\n",
+				usb_strerror());
 			exit(1);
 		}
 		goto status_again;
@@ -746,7 +777,8 @@ status_again:
 	case DFU_STATE_dfuUPLOAD_IDLE:
 		printf("aborting previous incomplete transfer\n");
 		if (dfu_abort(dif->dev_handle, dif->interface) < 0) {
-			fprintf(stderr, "can't send DFU_ABORT: %s\n", usb_strerror());
+			fprintf(stderr, "can't send DFU_ABORT: %s\n",
+				usb_strerror());
 			exit(1);
 		}
 		goto status_again;
@@ -765,8 +797,7 @@ status_again:
 				"descriptor: %s\n", usb_strerror());
 			transfer_size = page_size;
 		} else {
-			func_dfu.wTransferSize = LE2CPU16(func_dfu.wTransferSize);
-			transfer_size = func_dfu.wTransferSize;
+			transfer_size = get_int16_le(&func_dfu.wTransferSize);
 		}
 	}
 
